/****************************************************************************
** Meta object code from reading C++ file 'gesturemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/core/gesturemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gesturemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14GestureManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto GestureManager::qt_create_metaobjectdata<qt_meta_tag_ZN14GestureManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GestureManager",
        "D-Bus Interface",
        "org.deepin.dde.Gesture1",
        "GestureInfosChanged",
        "",
        "GestureActivated",
        "id",
        "triggerValue",
        "ListAllGestures",
        "QList<GestureInfo>",
        "ModifyGesture",
        "action",
        "GetGestureAvaiableActions",
        "actionType",
        "fingerNum",
        "onGestureConfigAdded",
        "GestureConfig",
        "config",
        "onGestureConfigChanged",
        "onGestureConfigRemoved",
        "onGestureActivated",
        "gestureId",
        "onHandlerAvailabilityChanged",
        "available"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'GestureInfosChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'GestureActivated'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(5, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QStringList, 7 },
        }}),
        // Slot 'ListAllGestures'
        QtMocHelpers::SlotData<QList<GestureInfo>()>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 9),
        // Slot 'ModifyGesture'
        QtMocHelpers::SlotData<bool(const QString &, const QStringList &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 6 }, { QMetaType::QStringList, 11 },
        }}),
        // Slot 'GetGestureAvaiableActions'
        QtMocHelpers::SlotData<QString(const QString &, int)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 13 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'onGestureConfigAdded'
        QtMocHelpers::SlotData<void(const GestureConfig &)>(15, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'onGestureConfigChanged'
        QtMocHelpers::SlotData<void(const GestureConfig &)>(18, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'onGestureConfigRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(19, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'onGestureActivated'
        QtMocHelpers::SlotData<void(const QString &)>(20, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'onHandlerAvailabilityChanged'
        QtMocHelpers::SlotData<void(bool)>(22, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<GestureManager, qt_meta_tag_ZN14GestureManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject GestureManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14GestureManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14GestureManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14GestureManagerE_t>.metaTypes,
    nullptr
} };

void GestureManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GestureManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->GestureInfosChanged(); break;
        case 1: _t->GestureActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: { QList<GestureInfo> _r = _t->ListAllGestures();
            if (_a[0]) *reinterpret_cast<QList<GestureInfo>*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->ModifyGesture((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->GetGestureAvaiableActions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->onGestureConfigAdded((*reinterpret_cast<std::add_pointer_t<GestureConfig>>(_a[1]))); break;
        case 6: _t->onGestureConfigChanged((*reinterpret_cast<std::add_pointer_t<GestureConfig>>(_a[1]))); break;
        case 7: _t->onGestureConfigRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->onGestureActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->onHandlerAvailabilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GestureConfig >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GestureConfig >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GestureManager::*)()>(_a, &GestureManager::GestureInfosChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GestureManager::*)(const QString & , const QStringList & )>(_a, &GestureManager::GestureActivated, 1))
            return;
    }
}

const QMetaObject *GestureManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GestureManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14GestureManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int GestureManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void GestureManager::GestureInfosChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GestureManager::GestureActivated(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
