/****************************************************************************
** Meta object code from reading C++ file 'keybindingmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/core/keybindingmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keybindingmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KeybindingManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto KeybindingManager::qt_create_metaobjectdata<qt_meta_tag_ZN17KeybindingManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KeybindingManager",
        "D-Bus Interface",
        "org.deepin.dde.Keybinding1",
        "ShortcutChanged",
        "",
        "id",
        "ShortcutInfo",
        "info",
        "ShortcutActivated",
        "triggerValue",
        "ShortcutRemoved",
        "KeyEvent",
        "pressed",
        "keystroke",
        "NumLockStateChanged",
        "state",
        "CapsLockStateChanged",
        "ListAllShortcuts",
        "QList<ShortcutInfo>",
        "ListShortcutsByApp",
        "appId",
        "ListShortcutsByCategory",
        "category",
        "ListCategories",
        "QList<CategoryInfo>",
        "GetShortcut",
        "GetShortcutCommand",
        "LookupConflictShortcut",
        "hotkey",
        "SearchShortcuts",
        "keyword",
        "ModifyHotkeys",
        "newHotkeys",
        "Disable",
        "AddCustomShortcut",
        "name",
        "action",
        "AddCustomShortcutWithConflict",
        "expectedConflictId",
        "ModifyCustomShortcut",
        "ModifyCustomShortcutWithConflict",
        "DeleteCustomShortcut",
        "BeginCapture",
        "timeoutMs",
        "EndCapture",
        "SwapHotkeys",
        "id1",
        "id2",
        "ReplaceHotkey",
        "targetId",
        "newHotkey",
        "conflictId",
        "ReloadConfigs",
        "Reset",
        "GetNumLockState",
        "GetCapsLockState",
        "SetNumLockState",
        "SetCapsLockState",
        "onKeyConfigAdded",
        "KeyConfig",
        "config",
        "onKeyConfigChanged",
        "onConfigRemoved",
        "onKeyActivated",
        "shortcutId",
        "onSpecialKeyActivated",
        "onCaptureKeyEvent",
        "updateNumLockState",
        "on",
        "updateCapsLockState",
        "onBackendKeymapAboutToChange",
        "onBackendKeymapChanged",
        "toShortcutInfo",
        "NumLockState",
        "CapsLockState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ShortcutChanged'
        QtMocHelpers::SignalData<void(const QString &, const ShortcutInfo &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'ShortcutActivated'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QStringList, 9 },
        }}),
        // Signal 'ShortcutRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'KeyEvent'
        QtMocHelpers::SignalData<void(bool, const QString &)>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'NumLockStateChanged'
        QtMocHelpers::SignalData<void(uint)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Signal 'CapsLockStateChanged'
        QtMocHelpers::SignalData<void(uint)>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Slot 'ListAllShortcuts'
        QtMocHelpers::SlotData<QList<ShortcutInfo>()>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 18),
        // Slot 'ListShortcutsByApp'
        QtMocHelpers::SlotData<QList<ShortcutInfo>(const QString &)>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 18, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'ListShortcutsByCategory'
        QtMocHelpers::SlotData<QList<ShortcutInfo>(const QString &)>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 18, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'ListCategories'
        QtMocHelpers::SlotData<QList<CategoryInfo>()>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 24),
        // Slot 'GetShortcut'
        QtMocHelpers::SlotData<ShortcutInfo(const QString &)>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 6, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'GetShortcutCommand'
        QtMocHelpers::SlotData<QString(const QString &)>(26, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'LookupConflictShortcut'
        QtMocHelpers::SlotData<ShortcutInfo(const QString &)>(27, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 6, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'SearchShortcuts'
        QtMocHelpers::SlotData<QList<ShortcutInfo>(const QString &)>(29, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 18, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'ModifyHotkeys'
        QtMocHelpers::SlotData<bool(const QString &, const QStringList &)>(31, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 }, { QMetaType::QStringList, 32 },
        }}),
        // Slot 'Disable'
        QtMocHelpers::SlotData<bool(const QString &)>(33, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'AddCustomShortcut'
        QtMocHelpers::SlotData<QString(const QString &, const QString &, const QString &)>(34, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 36 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'AddCustomShortcutWithConflict'
        QtMocHelpers::SlotData<QString(const QString &, const QString &, const QString &, const QString &)>(37, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 36 }, { QMetaType::QString, 28 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'ModifyCustomShortcut'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, const QString &)>(39, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 35 }, { QMetaType::QString, 36 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'ModifyCustomShortcutWithConflict'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, const QString &, const QString &)>(40, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 35 }, { QMetaType::QString, 36 }, { QMetaType::QString, 28 },
            { QMetaType::QString, 38 },
        }}),
        // Slot 'DeleteCustomShortcut'
        QtMocHelpers::SlotData<bool(const QString &)>(41, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'BeginCapture'
        QtMocHelpers::SlotData<bool(uint)>(42, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::UInt, 43 },
        }}),
        // Slot 'BeginCapture'
        QtMocHelpers::SlotData<bool()>(42, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, QMetaType::Bool),
        // Slot 'EndCapture'
        QtMocHelpers::SlotData<void()>(44, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'SwapHotkeys'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(45, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 46 }, { QMetaType::QString, 47 },
        }}),
        // Slot 'ReplaceHotkey'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &)>(48, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 49 }, { QMetaType::QString, 50 }, { QMetaType::QString, 51 },
        }}),
        // Slot 'ReloadConfigs'
        QtMocHelpers::SlotData<void()>(52, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'Reset'
        QtMocHelpers::SlotData<void()>(53, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'GetNumLockState'
        QtMocHelpers::SlotData<uint() const>(54, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'GetCapsLockState'
        QtMocHelpers::SlotData<uint() const>(55, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'SetNumLockState'
        QtMocHelpers::SlotData<void(uint)>(56, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Slot 'SetCapsLockState'
        QtMocHelpers::SlotData<void(uint)>(57, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Slot 'onKeyConfigAdded'
        QtMocHelpers::SlotData<void(const KeyConfig &)>(58, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'onKeyConfigChanged'
        QtMocHelpers::SlotData<void(const KeyConfig &)>(61, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'onConfigRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(62, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'onKeyActivated'
        QtMocHelpers::SlotData<void(const QString &)>(63, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Slot 'onSpecialKeyActivated'
        QtMocHelpers::SlotData<void(const QString &)>(65, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Slot 'onCaptureKeyEvent'
        QtMocHelpers::SlotData<void(bool, const QString &)>(66, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'updateNumLockState'
        QtMocHelpers::SlotData<void(bool)>(67, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 68 },
        }}),
        // Slot 'updateCapsLockState'
        QtMocHelpers::SlotData<void(bool)>(69, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 68 },
        }}),
        // Slot 'onBackendKeymapAboutToChange'
        QtMocHelpers::SlotData<void()>(70, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onBackendKeymapChanged'
        QtMocHelpers::SlotData<void()>(71, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toShortcutInfo'
        QtMocHelpers::SlotData<ShortcutInfo(const KeyConfig &)>(72, 4, QMC::AccessPrivate, 0x80000000 | 6, {{
            { 0x80000000 | 59, 60 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'NumLockState'
        QtMocHelpers::PropertyData<uint>(73, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'CapsLockState'
        QtMocHelpers::PropertyData<uint>(74, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<KeybindingManager, qt_meta_tag_ZN17KeybindingManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KeybindingManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KeybindingManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KeybindingManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KeybindingManagerE_t>.metaTypes,
    nullptr
} };

void KeybindingManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeybindingManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ShortcutChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ShortcutInfo>>(_a[2]))); break;
        case 1: _t->ShortcutActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: _t->ShortcutRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->KeyEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->NumLockStateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 5: _t->CapsLockStateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 6: { QList<ShortcutInfo> _r = _t->ListAllShortcuts();
            if (_a[0]) *reinterpret_cast<QList<ShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QList<ShortcutInfo> _r = _t->ListShortcutsByApp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<ShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QList<ShortcutInfo> _r = _t->ListShortcutsByCategory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<ShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QList<CategoryInfo> _r = _t->ListCategories();
            if (_a[0]) *reinterpret_cast<QList<CategoryInfo>*>(_a[0]) = std::move(_r); }  break;
        case 10: { ShortcutInfo _r = _t->GetShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ShortcutInfo*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->GetShortcutCommand((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { ShortcutInfo _r = _t->LookupConflictShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ShortcutInfo*>(_a[0]) = std::move(_r); }  break;
        case 13: { QList<ShortcutInfo> _r = _t->SearchShortcuts((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<ShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->ModifyHotkeys((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->Disable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { QString _r = _t->AddCustomShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->AddCustomShortcutWithConflict((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->ModifyCustomShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->ModifyCustomShortcutWithConflict((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->DeleteCustomShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->BeginCapture((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->BeginCapture();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->EndCapture(); break;
        case 24: { bool _r = _t->SwapHotkeys((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->ReplaceHotkey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->ReloadConfigs(); break;
        case 27: _t->Reset(); break;
        case 28: { uint _r = _t->GetNumLockState();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 29: { uint _r = _t->GetCapsLockState();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->SetNumLockState((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 31: _t->SetCapsLockState((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 32: _t->onKeyConfigAdded((*reinterpret_cast<std::add_pointer_t<KeyConfig>>(_a[1]))); break;
        case 33: _t->onKeyConfigChanged((*reinterpret_cast<std::add_pointer_t<KeyConfig>>(_a[1]))); break;
        case 34: _t->onConfigRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->onKeyActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->onSpecialKeyActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->onCaptureKeyEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 38: _t->updateNumLockState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->updateCapsLockState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->onBackendKeymapAboutToChange(); break;
        case 41: _t->onBackendKeymapChanged(); break;
        case 42: { ShortcutInfo _r = _t->toShortcutInfo((*reinterpret_cast<std::add_pointer_t<KeyConfig>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ShortcutInfo*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ShortcutInfo >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyConfig >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyConfig >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyConfig >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(const QString & , const ShortcutInfo & )>(_a, &KeybindingManager::ShortcutChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(const QString & , const QStringList & )>(_a, &KeybindingManager::ShortcutActivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(const QString & )>(_a, &KeybindingManager::ShortcutRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(bool , const QString & )>(_a, &KeybindingManager::KeyEvent, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(uint )>(_a, &KeybindingManager::NumLockStateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeybindingManager::*)(uint )>(_a, &KeybindingManager::CapsLockStateChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->GetNumLockState(); break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->GetCapsLockState(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->SetNumLockState(*reinterpret_cast<uint*>(_v)); break;
        case 1: _t->SetCapsLockState(*reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KeybindingManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeybindingManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KeybindingManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int KeybindingManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KeybindingManager::ShortcutChanged(const QString & _t1, const ShortcutInfo & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KeybindingManager::ShortcutActivated(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KeybindingManager::ShortcutRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KeybindingManager::KeyEvent(bool _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KeybindingManager::NumLockStateChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KeybindingManager::CapsLockStateChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
