/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.dde.WallpaperSlideshow.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "wallpaperslideshowadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class WallpaperSlideshowAdaptor
 */

WallpaperSlideshowAdaptor::WallpaperSlideshowAdaptor(WallpaperSlideshow *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

WallpaperSlideshowAdaptor::~WallpaperSlideshowAdaptor()
{
    // destructor
}

QString WallpaperSlideshowAdaptor::wallpaperSlideShow() const
{
    // get the value of property WallpaperSlideShow
    return qvariant_cast< QString >(parent()->property("WallpaperSlideShow"));
}

void WallpaperSlideshowAdaptor::setWallpaperSlideShow(const QString &value)
{
    // set the value of property WallpaperSlideShow
    parent()->setProperty("WallpaperSlideShow", QVariant::fromValue(value));
}

QString WallpaperSlideshowAdaptor::GetWallpaperSlideShow(const QString &monitorName)
{
    // handle method call org.deepin.dde.WallpaperSlideshow.GetWallpaperSlideShow
    return parent()->GetWallpaperSlideShow(monitorName);
}

void WallpaperSlideshowAdaptor::SetWallpaperSlideShow(const QString &monitorName, const QString &slideShow)
{
    // handle method call org.deepin.dde.WallpaperSlideshow.SetWallpaperSlideShow
    parent()->SetWallpaperSlideShow(monitorName, slideShow);
}


#include "moc_wallpaperslideshowadaptor.cpp"
