/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024-2025 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface treeland_output_color_control_v1_interface;
extern const struct wl_interface wl_output_interface;

static const struct wl_interface *treeland_output_manager_v1_types[] = {
	NULL,
	&treeland_output_color_control_v1_interface,
	&wl_output_interface,
};

static const struct wl_message treeland_output_manager_v1_requests[] = {
	{ "set_primary_output", "s", treeland_output_manager_v1_types + 0 },
	{ "get_color_control", "2no", treeland_output_manager_v1_types + 1 },
	{ "destroy", "2", treeland_output_manager_v1_types + 0 },
};

static const struct wl_message treeland_output_manager_v1_events[] = {
	{ "primary_output", "s", treeland_output_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_output_manager_v1_interface = {
	"treeland_output_manager_v1", 2,
	3, treeland_output_manager_v1_requests,
	1, treeland_output_manager_v1_events,
};

static const struct wl_message treeland_output_color_control_v1_requests[] = {
	{ "set_color_temperature", "u", treeland_output_manager_v1_types + 0 },
	{ "set_brightness", "f", treeland_output_manager_v1_types + 0 },
	{ "commit", "", treeland_output_manager_v1_types + 0 },
	{ "destroy", "", treeland_output_manager_v1_types + 0 },
};

static const struct wl_message treeland_output_color_control_v1_events[] = {
	{ "result", "u", treeland_output_manager_v1_types + 0 },
	{ "color_temperature", "u", treeland_output_manager_v1_types + 0 },
	{ "brightness", "f", treeland_output_manager_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_output_color_control_v1_interface = {
	"treeland_output_color_control_v1", 1,
	4, treeland_output_color_control_v1_requests,
	3, treeland_output_color_control_v1_events,
};

