/****************************************************************************
** Meta object code from reading C++ file 'xsettings1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../impl/xsettings1.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xsettings1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10XSettings1E_t {};
} // unnamed namespace

template <> constexpr inline auto XSettings1::qt_create_metaobjectdata<qt_meta_tag_ZN10XSettings1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XSettings1",
        "D-Bus Interface",
        "org.deepin.dde.XSettings1",
        "SetScaleFactorDone",
        "",
        "SetScaleFactorStarted",
        "GetColor",
        "ArrayOfColor",
        "prop",
        "GetInteger",
        "GetScaleFactor",
        "GetScreenScaleFactors",
        "ScaleFactors",
        "GetString",
        "ListProps",
        "SetColor",
        "v",
        "SetInteger",
        "SetScaleFactor",
        "scale",
        "SetScreenScaleFactors",
        "factors",
        "SetString"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SetScaleFactorDone'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SetScaleFactorStarted'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'GetColor'
        QtMocHelpers::SlotData<ArrayOfColor(const QString &)>(6, 4, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'GetInteger'
        QtMocHelpers::SlotData<qint32(const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'GetScaleFactor'
        QtMocHelpers::SlotData<double()>(10, 4, QMC::AccessPublic, QMetaType::Double),
        // Slot 'GetScreenScaleFactors'
        QtMocHelpers::SlotData<ScaleFactors()>(11, 4, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'GetString'
        QtMocHelpers::SlotData<QString(const QString &)>(13, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'ListProps'
        QtMocHelpers::SlotData<QString()>(14, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'SetColor'
        QtMocHelpers::SlotData<void(const QString &, const ArrayOfColor &)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 7, 16 },
        }}),
        // Slot 'SetInteger'
        QtMocHelpers::SlotData<void(const QString &, const qint32 &)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'SetScaleFactor'
        QtMocHelpers::SlotData<void(const double &)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 19 },
        }}),
        // Slot 'SetScreenScaleFactors'
        QtMocHelpers::SlotData<void(const ScaleFactors &)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 21 },
        }}),
        // Slot 'SetString'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<XSettings1, qt_meta_tag_ZN10XSettings1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject XSettings1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10XSettings1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10XSettings1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10XSettings1E_t>.metaTypes,
    nullptr
} };

void XSettings1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XSettings1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SetScaleFactorDone(); break;
        case 1: _t->SetScaleFactorStarted(); break;
        case 2: { ArrayOfColor _r = _t->GetColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ArrayOfColor*>(_a[0]) = std::move(_r); }  break;
        case 3: { qint32 _r = _t->GetInteger((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint32*>(_a[0]) = std::move(_r); }  break;
        case 4: { double _r = _t->GetScaleFactor();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 5: { ScaleFactors _r = _t->GetScreenScaleFactors();
            if (_a[0]) *reinterpret_cast<ScaleFactors*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->GetString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->ListProps();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->SetColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ArrayOfColor>>(_a[2]))); break;
        case 9: _t->SetInteger((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint32>>(_a[2]))); break;
        case 10: _t->SetScaleFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->SetScreenScaleFactors((*reinterpret_cast<std::add_pointer_t<ScaleFactors>>(_a[1]))); break;
        case 12: _t->SetString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (XSettings1::*)()>(_a, &XSettings1::SetScaleFactorDone, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (XSettings1::*)()>(_a, &XSettings1::SetScaleFactorStarted, 1))
            return;
    }
}

const QMetaObject *XSettings1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XSettings1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10XSettings1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int XSettings1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void XSettings1::SetScaleFactorDone()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void XSettings1::SetScaleFactorStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
